package rsg.nl.adf;

import java.io.BufferedOutputStream;
import java.io.File;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

import java.io.File;
import java.io.IOException;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;

import javax.xml.transform.Result;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import javax.xml.transform.stream.StreamSource;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Attr;

import org.xml.sax.SAXException;

public class JSPXProcessor {

    private static String baseDirectory =
        "D:\\Repos\\mwp2\\mwp-web\\public_html";

    private final static List<String> elementNamesThatNeedId =
        Arrays.asList("outputText", "inputText", "panelGroup", "popup",
                      "iterator", "noteWindow", "cellFormat", "rowLayout",
                      "tableLayout", "spacer", "switcher", "showDetailItem",
                      "panelTabbed", "commandLink", "image", "group",
                      "notificatie", "panelBox", "panelGroupLayout",
                      "outputFormatted", "meldingen", "pageTemplate");



    public static void processJSPX(File inputFile, int fileSequence) throws SAXException,
                                                          IOException,
                                                          ParserConfigurationException,
                                                          TransformerConfigurationException,
                                                          TransformerException,
                                                          XPathExpressionException {
        DocumentBuilderFactory dbFactory =
            DocumentBuilderFactory.newInstance();
        DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
        Document doc = dBuilder.parse(inputFile);
        Boolean changed = traverseNodes(doc, doc, 0, fileSequence);

        if (changed) {
        // Set up output sink
        Result outputFile = new StreamResult(inputFile);

        // Setup a factory for transforms
        TransformerFactory factory = TransformerFactory.newInstance();

        // Get a transformer for this XSL
        Transformer transformer = factory.newTransformer();
        Source docSource = new DOMSource(doc);
        System.out.println("- gewijzigd en weggeschreven");
        //Result streamResult = new StreamResult(System.out);
        transformer.transform(docSource, outputFile);
        }
    }

    public static Boolean traverseNodes(Document d, Node n,
                                     int uniqueCounter, int fileSequence) throws XPathExpressionException {
        
        Boolean changed = false;
        NodeList children = n.getChildNodes();
        if (children != null) {
            for (int i = 0; i < children.getLength(); i++) {
                Node childNode = children.item(i);

                // here would be a good place to put your application logic
                // and do something base upon node type
                if (childNode.getNodeType() == Node.ELEMENT_NODE) {
                    Element e = (Element)childNode;
                    Attr id = e.getAttributeNode("id");
                    String idValue = (id == null ? "null" : id.getNodeValue());
                    if (id == null &&
                        elementNamesThatNeedId.contains(childNode.getLocalName())) {
                        while (true) {
                            uniqueCounter++;
                            idValue =
                                    childNode.getLocalName().substring(0, 1) +
                                    uniqueCounter + "g"+fileSequence;
                            if (checkUniquenessOfIdAttribute(d, idValue)) {
                                break;
                            }
                        }
                        e.setAttribute("id", idValue);
                        changed=true;
                        System.out.println(" - Add id value " + idValue +
                                           " to element " +
                                           childNode.getNodeName());
                    }

                }
                changed = changed | traverseNodes(d, childNode, uniqueCounter, fileSequence);
            }
        }
        return changed;
    }

    private static boolean checkUniquenessOfIdAttribute(Document d,
                                                        String idValue) throws XPathExpressionException {
        XPathFactory factory = XPathFactory.newInstance();
        XPath xpath = factory.newXPath();
        XPathExpression expr = xpath.compile("//*[@id='" + idValue + "']");

        NodeList result = (NodeList)expr.evaluate(d, XPathConstants.NODESET);
        return result.getLength() == 0;
    }

    public static int scanFilesRecursively(File file, int fileSequence) throws SAXException,
                                                              IOException,
                                                              ParserConfigurationException,
                                                              TransformerConfigurationException,
                                                              TransformerException,
                                                              XPathExpressionException {
        final File[] children = file.listFiles();
        if (children != null) {
            for (File child : children) {
               
                String filename = child.getName();
                if (filename.endsWith("jsff") || filename.endsWith("jspx")) {
                   // if (filename.startsWith("essPersonalia.jsff")) {
                        System.out.println("Processing " + child.getName() +
                                           " in " + child.getAbsolutePath());
                         fileSequence++;
                         processJSPX(child, fileSequence);
                    }
              //  }
                fileSequence = scanFilesRecursively(child,fileSequence);
            }
        }
        return fileSequence;
    }

    public static void main(String[] args) throws SAXException, IOException,
                                                  ParserConfigurationException,
                                                  TransformerConfigurationException,
                                                  TransformerException,
                                                  XPathExpressionException {

        int numberOfFilesProcessed = scanFilesRecursively(new File(baseDirectory),0);
        System.out.println("Aantal files gescand = "+numberOfFilesProcessed);
        //        String fileName = "secure\\urenEnVergoedingen\\uren.jsff";
        //        File file = new File(baseDirectory + "\\" + fileName);
        //        processJSPX(file);
    }
}
